@@ -1,4 +1,4 @@
-// 
+//
 // TrackFilterType.cs
 //
 // Author:
@@ -38,6 +38,8 @@ namespace Banshee.Collection
         ArtistName = 1 << 1,
         AlbumTitle = 1 << 2,
         Genre      = 1 << 3,
-        Year       = 1 << 4
+        Year       = 1 << 4,
+        Comment    = 1 << 5,
+        AlbumArtist= 1 << 6,
     }
 }
