@@ -30,6 +30,8 @@ using System;
 using System.IO;
 using System.Collections.Generic;
 
+using Hyena;
+
 using TagLib;
 
 using Banshee.Base;
@@ -88,8 +90,9 @@ namespace Banshee.Metadata.Embedded
 
         protected IPicture [] GetEmbeddedPictures(SafeUri uri)
         {
-            TagLib.File file = StreamTagger.ProcessUri(uri);
-            return file == null ? null : file.Tag.Pictures;
+            using (var file = StreamTagger.ProcessUri (uri)) {
+                return file == null ? null : file.Tag.Pictures;
+            }
         }
 
         protected int GetPictureIndexToUse(IPicture [] pictures)
