@@ -68,13 +68,13 @@ namespace Banshee.Podcasting.Gui
             AddImportant (
                 new ActionEntry (
                     "PodcastUpdateAllAction", Stock.Refresh,
-                     Catalog.GetString ("Check for New Episodes"), null,//"<control><shift>U",
-                     Catalog.GetString ("Refresh All Podcasts"),
+                     Catalog.GetString ("Refresh"), null,//"<control><shift>U",
+                     Catalog.GetString ("Check all podcasts for new episodes"),
                      OnPodcastUpdateAll
                 ),
                 new ActionEntry (
                     "PodcastAddAction", Stock.Add,
-                     Catalog.GetString ("Subscribe to Podcast..."),"<control><shift>F",
+                     Catalog.GetString ("Add Podcast..."),"<control><shift>F",
                      Catalog.GetString ("Subscribe to a new podcast"),
                      OnPodcastAdd
                 )
@@ -158,7 +158,7 @@ namespace Banshee.Podcasting.Gui
                 )
             );
 
-            this["PodcastAddAction"].ShortLabel = Catalog.GetString ("Subscribe to Podcast");
+            this["PodcastAddAction"].ShortLabel = Catalog.GetString ("Add Podcast");
 
             actions_id = Actions.UIManager.AddUiFromResource ("GlobalUI.xml");
             Actions.AddActionGroup (this);
@@ -298,33 +298,35 @@ namespace Banshee.Podcasting.Gui
         {
             Uri feedUri = null;
             FeedAutoDownload syncPreference;
+            string url = null;
 
             PodcastSubscribeDialog subscribeDialog = new PodcastSubscribeDialog ();
 
             ResponseType response = (ResponseType) subscribeDialog.Run ();
+
             syncPreference = subscribeDialog.SyncPreference;
 
+            if (response == ResponseType.Ok) {
+                url = subscribeDialog.Url.Trim ().Trim ('/');
+            }
+
             subscribeDialog.Destroy ();
 
-            if (response == ResponseType.Ok) {
-                string url = subscribeDialog.Url.Trim ().Trim ('/');
+            if (String.IsNullOrEmpty (url)) {
+                return;
+            }
 
-                if (String.IsNullOrEmpty (subscribeDialog.Url)) {
-                    return;
-                }
-
-				if (!TryParseUrl (url, out feedUri)) {
-                    HigMessageDialog.RunHigMessageDialog (
-                        null,
-                        DialogFlags.Modal,
-                        MessageType.Warning,
-                        ButtonsType.Ok,
-                        Catalog.GetString ("Invalid URL"),
-                        Catalog.GetString ("Podcast URL is invalid.")
-                    );
-				} else {
-				    SubscribeToPodcast (feedUri, syncPreference);
-				}
+            if (!TryParseUrl (url, out feedUri)) {
+                HigMessageDialog.RunHigMessageDialog (
+                    null,
+                    DialogFlags.Modal,
+                    MessageType.Warning,
+                    ButtonsType.Ok,
+                    Catalog.GetString ("Invalid URL"),
+                    Catalog.GetString ("Podcast URL is invalid.")
+                );
+            } else {
+                SubscribeToPodcast (feedUri, syncPreference);
             }
         }
 
@@ -443,7 +445,9 @@ namespace Banshee.Podcasting.Gui
         private void OnPodcastUpdateAll (object sender, EventArgs e)
         {
             foreach (Feed feed in Feed.Provider.FetchAll ()) {
-                feed.Update ();
+                if (feed.IsSubscribed) {
+                    feed.Update ();
+                }
             }
         }
 
